import { Hash, LocationDescriptorObject, Pathname, Search, UnregisterCallback } from 'history';
import { ComponentType, ReactChild } from 'react';
export declare type HistoryAdapter = {
    location: {
        pathname: Pathname;
        search: Search;
        hash: Hash;
    };
    listen: (listener: () => void) => UnregisterCallback;
    push: (location: LocationDescriptorObject) => void;
    replace: (location: LocationDescriptorObject) => void;
};
export declare const html5HistoryAdapter: HistoryAdapter;
export declare type Config<T> = {
    history: HistoryAdapter;
    serialisation: {
        parse: (queryString: Search) => T;
        stringify: (state: T) => Search;
    };
    shouldPushState: (next: T, current: T) => boolean;
};
export declare function useUrlState<T>(initialState: T, config?: Partial<Config<T>>): [T, (newState: T) => void];
export declare type PropEnhancer<Props, MappedProps> = (component: ComponentType<MappedProps>) => ComponentType<Props>;
export declare type HocConfig<T, OP> = {
    history: HistoryAdapter;
    serialisation: {
        parse: (queryString: string) => T;
        stringify: (state: T) => string;
    };
    shouldPushState: (props: OP) => (next: T, current: T) => boolean;
};
export declare type UrlStateProps<T> = {
    setUrlState: (newState: T) => void;
    urlState: T;
};
export declare const withUrlState: <T extends {}, OP = {}>(getInitialState: (props: OP) => T, config?: Partial<HocConfig<T, OP>>) => PropEnhancer<OP, OP & UrlStateProps<T>>;
export declare type Props<T> = {
    config?: Config<T>;
    initialState: T;
    render: (renderProps: UrlStateProps<T>) => ReactChild;
};
export declare const UrlState: <T>(props: Props<T>) => ReactChild;
