"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var qs_1 = __importDefault(require("qs"));
var react_1 = require("react");
exports.html5HistoryAdapter = {
    listen: function (listener) {
        window.addEventListener('popstate', listener);
        return function () { return window.removeEventListener('popstate', listener); };
    },
    location: window.location,
    push: function (_a) {
        var search = _a.search;
        window.history.pushState(window.history.state, document.title, search);
        window.dispatchEvent(new window.Event('popstate'));
    },
    replace: function (_a) {
        var search = _a.search;
        window.history.replaceState(window.history.state, document.title, search);
        window.dispatchEvent(new window.Event('popstate'));
    },
};
var alwaysReplaceState = function () { return false; };
var parseConfig = function (config) {
    if (config === void 0) { config = {}; }
    return ({
        history: config.history || exports.html5HistoryAdapter,
        serialisation: config.serialisation || {
            parse: function (queryString) { return qs_1.default.parse(queryString, { ignoreQueryPrefix: true }); },
            stringify: function (state) { return qs_1.default.stringify(state, { addQueryPrefix: true }); },
        },
        shouldPushState: config.shouldPushState || alwaysReplaceState,
    });
};
function useUrlState(initialState, config) {
    var _a = parseConfig(config), history = _a.history, serialisation = _a.serialisation, shouldPushState = _a.shouldPushState;
    // tslint:disable:no-any Typescript cant handle generic spread yet
    var _b = react_1.useState(__assign({}, initialState, serialisation.parse(history.location.search))), currentState = _b[0], setSearch = _b[1];
    // tslint:enable:no-any
    react_1.useEffect(function () {
        history.replace(__assign({}, history.location, { search: serialisation.stringify(currentState) }));
        return history.listen(function onLocationChange() {
            setSearch(serialisation.parse(history.location.search));
        });
    }, [history]);
    function setUrlState(newState) {
        var nextLocation = __assign({}, history.location, { search: serialisation.stringify(newState) });
        shouldPushState(newState, currentState)
            ? history.push(nextLocation)
            : history.replace(nextLocation);
    }
    return [currentState, setUrlState];
}
exports.useUrlState = useUrlState;
exports.withUrlState = function (getInitialState, config) {
    if (config === void 0) { config = {}; }
    return function (WrappedComponent) { return function (props) {
        var hocConfig = __assign({}, config, { shouldPushState: config.shouldPushState && config.shouldPushState(props) });
        var _a = useUrlState(getInitialState(props), hocConfig), urlState = _a[0], setUrlState = _a[1];
        return react_1.createElement(WrappedComponent, __assign({}, props, { urlState: urlState,
            setUrlState: setUrlState }));
    }; };
};
exports.UrlState = function (props) {
    var _a = useUrlState(props.initialState, props.config), urlState = _a[0], setUrlState = _a[1];
    return props.render({ setUrlState: setUrlState, urlState: urlState });
};
